/*
	Copyright (C) 2013 Lukasz Nidecki SQ5RWU

    This file is part of ArduinoQAPRS.

    ArduinoQAPRS is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    ArduinoQAPRS is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with ArduinoQAPRS; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

    Ten plik jest cz??ci? ArduinoQAPRS.

    ArduinoQAPRS jest wolnym oprogramowaniem; mo?esz go rozprowadza? dalej
    i/lub modyfikowa? na warunkach Powszechnej Licencji Publicznej GNU,
    wydanej przez Fundacj? Wolnego Oprogramowania - wed?ug wersji 2 tej
    Licencji lub (wed?ug twojego wyboru) ktrej? z p?niejszych wersji.

    Niniejszy program rozpowszechniany jest z nadziej?, i? b?dzie on
    u?yteczny - jednak BEZ JAKIEJKOLWIEK GWARANCJI, nawet domy?lnej
    gwarancji PRZYDATNO?CI HANDLOWEJ albo PRZYDATNO?CI DO OKRE?LONYCH
    ZASTOSOWA?. W celu uzyskania bli?szych informacji si?gnij do
    Powszechnej Licencji Publicznej GNU.

    Z pewno?ci? wraz z niniejszym programem otrzyma?e? te? egzemplarz
    Powszechnej Licencji Publicznej GNU (GNU General Public License);
    je?li nie - napisz do Free Software Foundation, Inc., 59 Temple
    Place, Fifth Floor, Boston, MA  02110-1301  USA
 */

/* @file  * @brief Nag?wki dla klasy QAPRSR2R  */

#ifndef QAPRSR2R_H_
#define QAPRSR2R_H_
#include "QAPRSBase.h"

/* @brief R2R AFSK @details Klasa generuj?ca AFSK na drabince r2r na porcie B  */

class QAPRSR2R: public QAPRSBase {
private:
	volatile uint16_t timer1StartValue;

/* @brief Sta?a czasowa dla timera pomiedzy kolejnymi samplami sinusa dla Mark */

#if F_CPU == 16000000L
	static const uint16_t toneMarkTimeVHF = 65535 - 640;
#elif F_CPU == 8000000UL
	static const uint16_t toneMarkTimeVHF = 65535 - 245;
#endif

/* @brief Sta?a czasowa dla timera pomiedzy kolejnymi samplami sinusa dla Space  */

#if F_CPU == 16000000L
	static const uint16_t toneSpaceTimeVHF =  65535 - 363;
#elif F_CPU == 8000000UL
	static const uint16_t toneSpaceTimeVHF =  65535 - 160;
#endif

/* @brief Sta?a czasowa dla timera pomiedzy kolejnymi samplami sinusa dla Mark */

	static const uint16_t toneMarkTimeHF = 65535 - (596);

/* @brief Sta?a czasowa dla timera pomiedzy kolejnymi samplami sinusa dla Space  */

	static const uint16_t toneSpaceTimeHF =  65535 - (530);
	uint16_t toneMarkTime;
	uint16_t toneSpaceTime;
	virtual void enableTransmission();
	virtual void disableTranssmision();
	virtual uint8_t canTransmit();
protected:
	void toggleTone();
public:
	QAPRSR2R() {} ;
	virtual void timerInterruptHandler();
	virtual void init(int8_t sensePin, int8_t txEnablePin);
	using QAPRSBase::init;
	virtual void setVariant(QAPRSVariant variant = QAPRSVHF);
};


#endif /* QAPRSR2R_H_ */