// simple APRS tracker; original by Lukasz Nidecki, SQ5RWU, modified/extended by DJ7OO
//
// #1 (TxD) : output for 9600bps frequency control DRA818V
// #2       : input for transmit blocking ( active blocking if "L" )  
// #3       : output for PTT control ( active = "H" ) 
// #4- #7   : R2R network ( with using modified R2R library )  
// #A0      : input for determing transmit intervals ( 0V = 10sec.; 5V = 600sec. )  
// #A1      : "L" for setting to test mode with transmit intervals = 10sec.
// #A2      : input for DRA818 frequency selection; "H"= 144.800 MHz; "L"= 145.200 MHz 
// #A3      : input for detecting supply voltage
// #A4      : output for 9600bps data ( currently not used ) 
// #A5      : input for 9600bps gps NMEA data ( using GGA protocols ) 
 
 #include <string.h>
 #include <Arduino.h>
 #include <ArduinoQAPRS.h>
 #include <TinyGPS++.h>

TinyGPSPlus gps;   // The TinyGPS++ object

char * packet_buffer  = "                                                                    \n ";
char floatbuf[32];
char gradbuf[4];

String outString = "";
int ledPin = 13;                  // LED test pin
int alti;
boolean stringComplete = false;
float BattVolts; 
String text = "Klaus JN49CX";     // change according to your needs

///////////////////////////////////////////////////////////////////////////////////////
void btrd()
{
  analogReference(INTERNAL); 
  float BattRead = analogRead(A3);
//  Serial.println(BattRead);
  BattVolts = BattRead/163;
}

//////////////////////////////////////////////////////
void interval()
{
// variable delay depending on #A0 analog value 
   int val,z,calc;
   val = analogRead(A0);  // read analog value from poti ( 0 to 1023 )
   calc = map(val,0,1023,10,600);
//   calc = 10;
   Serial.print("Delay:   "),Serial.print(calc),Serial.println(" Secs.");
   for(z=1; z<(calc); z++)  
 { delay(1000); }         // delay = 10-600 Secs. (10 Sec. to 10 Min.)
}

///////////////////////////////////////////////////////// 
void setup()
{
  Serial.begin(9600);            // data rate GPS receiver
  Serial.println("WAIT!");
  pinMode(15, INPUT_PULLUP);     // #15 = #A1 for test mode selection
  pinMode(16, INPUT_PULLUP);     // #16 = #A2 input for frq selection

// frequency selection via #A2 "H"= 144.800 MHz "L"= 145.200 MHz  
  int sensorVal2 = digitalRead(16);  
  delay(1000);
  if (sensorVal2 == HIGH) 
{ Serial.println("AT+DMOSETGROUP=0,144.8000,144.8000,0000,0,0000"); }  
  else
{ Serial.println("AT+DMOSETGROUP=0,145.2000,145.2000,0000,0,0000"); }  
  delay(1000);
//  Serial.println("Init QAPRS");

//  Pin 3 as input: ground for operation 
//  Pin 2 as output: PTT ctrl.

  QAPRS.init(3,2,"DJ7OO", '5', "APRS", '0', "WIDE1-1");
  delay(1000);
  pinMode(ledPin, OUTPUT);       // initialize LED pin
}

/////////////////////////////////////////////////////////////////////////////////////////
void displayInfo()
{
  String Ns, Ew;
  float Tlat, Tlon;
  int Talt;
  float Lat;
  float Lon;
  btrd();
  
//  Serial.print("Lat is "); Serial.println(gps.location.lat(),5);
//  Serial.print("Lon is "); Serial.println(gps.location.lng(),5);

// Serial.println(gps.time.value());
  
  Tlat=gps.location.lat();
  Tlon=gps.location.lng();
  Talt=gps.altitude.meters();

//  Tlat=(49.12345); //***************
//  Tlon=(8.12345);  //***************
//  Talt=(12345);    //***************
  delay(100);
  
//  Serial.print("Tlat is "); Serial.println(Tlat,5);
//  Serial.print("Tlon is "); Serial.println(Tlon,5);
//  Serial.print("Talt is "); Serial.println(Talt);
  
  if(Tlat<0) { Ns = "S"; } else { Ns = "N"; } 
  if(Tlon<0) { Ew = "W"; } else { Ew = "E"; } 
  if(Tlat < 0) { Tlat= -Tlat; }
  unsigned int Deg_Lat = Tlat; 
  Lat = 100*(Deg_Lat) + (Tlat - Deg_Lat)*60; 
  
  if(Tlon < 0) { Tlon= -Tlon; }
  unsigned int Deg_Lon = Tlon; 
  Lon = 100*(Deg_Lon) + (Tlon - Deg_Lon)*60; 
  
//  Serial.print("Lat is "); Serial.println(Lat,5);
//  Serial.print("Lon is "); Serial.println(Lon,5);

  outString += "!";
  if(Tlat==0) {outString += "000"; }
  if(Tlat>0&&Tlat<10) {outString += "0";}
  outString += String(Lat,2);    
  outString += Ns;    
  outString += "/";   
  if(Tlon==0) {outString += "0000"; }
  if(Tlon>0&&Tlon<100) {outString += "0";}
  if(Tlon>0&&Tlon<10) {outString += "0";}
  outString += String(Lon,2); 
  outString += Ew;
  outString += ">/A=";    
  if(Talt==0) { outString += "0000"; }
  if(Talt>0&&Talt<10000) {outString += "0";}
  if(Talt>0&&Talt<1000) {outString += "0";}
  if(Talt>0&&Talt<100) {outString += "0";}
  outString += Talt; 
  outString += "m "; 
  outString += String(BattVolts,2); 
  outString += ("V ");
  outString += text;   
// Serial.println(outString);

// convert string to char_array
   int str_len = outString.length() + 1; 
   char char_array[str_len];
   outString.toCharArray(char_array, str_len);

// send via packet_buffer 
   packet_buffer = char_array;
   Serial.println(packet_buffer); 
   QAPRS.sendData(packet_buffer);            // send data 
   outString = "";
   packet_buffer = "";

//  normal delay if #A1 = "HIGH" else delay 10 Sec, ( Test-Mode ) 
   int sensorVal1 = digitalRead(15);  
   if (sensorVal1 == HIGH) 
  { interval(); }
  else
  { delay(10000); }  // Test-Mode Delay 10 Secs.           
 }

/////////////////////////////////////////////////////////// 
void loop()
{
//  Serial.println("*");
  while (Serial.available() > 0)
    if (gps.encode(Serial.read()))
    {
      displayInfo(); 
    }
  if (millis() > 5000 && gps.charsProcessed() < 10)
  {
    digitalWrite(ledPin, HIGH);
    Serial.println("No GPS detected!");
    while (true);
  }
}  


