// simple APRS WX tracker using (modified) QAPRS Library by Lukasz Nidecki, SQ5RWU
// program code extended by DJ7OO

// #1 (TxD) : output for 9600bps frequency control DRA818V
// #2       : output for PTT control ( active = "H" ); also used for TX-Led 
// #3       : input for transmit blocking ( operation = "L" )  
// #4-#7    : R2R network ( with using modified R2R library )  
// #A0      : input for setting transmit intervals ( 0V = 10sec.; 5V = 600sec. )  
// #A1      : "L" input for 10 sec. test mode 
// #A2      : input for DRA818 frequency selection; "H"= 144.800 MHz; "L"= 145.200 MHz 
// #A3      : input for detecting supply voltage
// #A4      : BME280 SDA
// #A5      : BME280 SCL

#include <Arduino.h>
#include <ArduinoQAPRS.h>
#include "SparkFunBME280.h"
#include "Wire.h"

BME280 mySensor;

 String lat = "4957.60N";        // select: latitude   DDMM.MM  + hemisphere: N/S
 String lon = "00812.00E";       // select: longitude  DDDMM.MM + hemisphere: E/W
 String outstring = "";
 char * packet_buffer  = "                                                                    \n ";
 float BattVolts; 

//////////////////////////////////////////////////////////////////////////////////
void setup(){
  pinMode(15, INPUT_PULLUP); // #15 = A1 as input
  pinMode(16, INPUT_PULLUP); // #16 = #A2 input for frq selection
  
  Serial.begin(9600);
  Serial.println("Init DRA818");
  delay(1000);
 // frequency selection via #A2 "H"= 144.800 MHz "L"= 145.200 MHz  
  int sensorVal2 = digitalRead(16);  
  if (sensorVal2 == HIGH) 
{ Serial.println("AT+DMOSETGROUP=0,144.8000,144.8000,0000,0,0000"); }  
  else
{ Serial.println("AT+DMOSETGROUP=0,145.2000,145.2000,0000,0,0000"); }  
  delay(1000);
  Serial.println("Init BME280");
  delay(1000);
  Serial.println("Init QAPRS");

//  Pin 3 als Eingang: Sendesperre wenn "H"
//  Pin 2 als Ausgang: Sendesignalisierung ( PTT )
  QAPRS.init(3,2,"DJ7OO", '2', "APRSWX", '0', "WIDE2-2");
  
  mySensor.settings.commInterface = I2C_MODE;
  mySensor.settings.I2CAddress = 0x77;
  
  mySensor.settings.runMode = 3; //  3, Normal mode
  mySensor.settings.tStandby = 0; //  0, 0.5ms
  mySensor.settings.filter = 0; //  0, filter off
  mySensor.settings.tempOverSample = 1;
  mySensor.settings.pressOverSample = 1;
  mySensor.settings.humidOverSample = 1;
  Serial.println(mySensor.begin(), HEX);
}

/////////////////////////////////////////////////////////////////////////////////
void loop() {
// combining part strings for APRS TX-String 
  outstring = "!" + lat + "/" + lon;
  outstring = outstring + "_000/000g000t";   
  int long temp (mySensor.readTempF());
  if(temp<100) {outstring = outstring + "0";}
  if(temp<10) {outstring = outstring + "0";}
  outstring = outstring + (String(temp));
  outstring = outstring + ("h");
  int long humi (mySensor.readFloatHumidity()); 
  outstring = outstring + (humi);
  outstring = outstring + ("b");  
  int long pres (mySensor.readFloatPressure());
  pres = pres/100;
  if(pres<1000) { outstring = outstring + "0"; } 
  outstring = outstring + (pres);
  outstring = outstring + "0a";
  int long alti (mySensor.readFloatAltitudeMeters());
  outstring = outstring + (alti); 
  outstring = outstring + ("m "); 
  btrd();
  outstring = outstring + (BattVolts);
  outstring = outstring + ("V_WX-Mainz");
     
// convert string to char_array
   int str_len = outstring.length() + 1; 
   char char_array[str_len];
   outstring.toCharArray(char_array, str_len);    

// send via packet_buffer 
   packet_buffer = char_array;
   QAPRS.sendData(packet_buffer);
   Serial.println(packet_buffer);

// normal delay if #A1 = "HIGH" else delay 10 Sec, ( Test-Mode ) 
  int sensorVal1 = digitalRead(15);  
  if (sensorVal1 == HIGH) 
  { interval(); }
  else
  { delay(10000);}  // Test-Mode Delay 10 Secs.        
}

///////////////////////////////////////////////////////////////////////////////////////
void btrd()
{
  analogReference(INTERNAL); 
  float BattRead = analogRead(A3);
//  Serial.println(BattRead);
  BattVolts = BattRead/163;
}

///////////////////////////////////////////////////////////////////////
void interval()
   {
// variable delay depending on #A0 analog value 
   int val,z,calc;
   val = analogRead(A0);  // read analog value from poti ( 0 to 1023 )
   calc = map(val,0,1023,60,900);
// calc = 300;     // 300 = 300 Sec. ; for testing only 
// Serial.print("Delay:   "),Serial.print(calc),Serial.println(" Secs.");
   for(z=1; z<(calc); z++)  
 { delay(1000); }         // delay = 60-900 Secs. (1 Min. to 15 Min.)
   }
 //////////////////////////////////////////////////////////////////////


